#!/usr/bin/env python3
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Nicolas Pécheux <info.cpge@cpge.info>
# Sunday, 15 October 2018
# http://cpge.info
#::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

"""Vérification du DM n°3."""

import unittest


RUN_PYLINT = True


def tous_tests():
    """Effectue les tests."""
    total_general = 0
    success_general = 0
    log = []
    for test in sorted(TESTS.keys()):
        s = unittest.TestLoader().loadTestsFromTestCase(TESTS[test])
        with open(os.devnull, 'w', encoding="utf8") as null:
            res = unittest.TextTestRunner(stream=null).run(s)
        total = res.testsRun
        failed = len(res.failures)
        errors = len(res.errors)
        success = total - failed - errors
        log.append("{}/{} {}".format(success, total, test))
        total_general += total
        success_general += success
    print('=' * 40)
    print('\n'.join(log))
    print('=' * 40)
    print('Tests réussis : {}/{}'.format(success_general, total_general))


if __name__ == '__main__':

    import sys
    import os
    import os.path as osp
    import importlib

    # Vérifier que le code source de l'élève est bien là
    if not osp.exists("dm3.py"):
        raise FileNotFoundError(
            "Le code source dm3.py n'a pas été trouvé. Vérifier que votre "
            "programme est bien dans le même répertoire que ce script, qu'il "
            "s'appelle bien dm3.py et que vous avez utilisé l'exécution en "
            "tant que script sous Pyzo.")

    if RUN_PYLINT:
  
        try:
            # Importer pylint
            import pylint.lint as lint
            # Vérifier que le fichier de configuation de PyLint est bien là.
            if not osp.exists("pylint.rc"):
                raise FileNotFoundError(
                    "Le fichier de configuration pylint.rc n'a pas été trouvé. "
                    "Vérifier qu'il est bien dans le même répertoire que ce "
                    "script.")

            # Vérification du code source de l'élève
            try:
                results = lint.Run(["dm3.py", "--rcfile=pylint.rc"],
                                   do_exit=False)
            except:
                try:
                    # Anciennes versions de PyLint
                    results = lint.Run(["dm3.py", "--rcfile=pylint.rc"],
                                       exit=False)
                except:
                    sys.stderr.write(
                        "PyLint n'a pas fonctionné correctement.\n")
              
        except ImportError:
            sys.stderr.write(
                "PyLint n'est pas installé. Vous n'avez pas accès à "
                "l'évaluation.\n\nPour l'installer, dans la console IPython "
                "et avec une connexion internet, entrer : \n"
                "\n    pip install pylint\n\n")

    # Importer le code source de l'élève
    try:
        if "dm3" not in sys.modules:
            _P = importlib.import_module("dm3")
        else:
            _P = importlib.reload(_P)
    except:
        raise InterruptedError(
            "Erreur : votre programme source ne compile pas !")

    # Importer les tests
    if not osp.exists("tests_dm3.py"):
        raise FileNotFoundError(
            "Le fichier tests_dm3.py n'a pas été trouvé. Vérifier qu'il se "
            "trouve bien dans le même répertoire que ce script.")

    with open("tests_dm3.py", "rt", encoding="utf8") as tests_file:
        exec(tests_file.read())

    tous_tests()
